/*   IRC Services2
*   Copyright (c) 2003 Services Developers
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of version 1 of the GNU General Public License,
*  as published by the Free Software Foundation.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. The GNU General Public License
*  contains more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Id: resv.c,v 1.8 2003/11/25 01:23:28 kllrloop Exp $
*/

#include "config.h" /* Always put config.h first */
#include "irc_types.h"
#include "oline.h"
#include "privs.h"
#include "send.h"
#include "me.h"
#include "dcc.h"
#include "net.h"
#include "operserv.h"  
#include "main.h"
#include "protected.h"
#include "db.h"
#include "conf.h"
#include "irc_string.h"
#include "defines.h"
#include "client.h"
#include "mem.h"
#include "resv.h"
#include "replies.h"
#include "dcc.h"

reserve_t *main_reserve = NULL;

reserve_type reserve_types[] = {
                                 { "COUNT", RESERVE_CNT },         /* To list count */  
                                 { "ALL", RESERVE_ALL   },         /* To list all RESV's */
                                 { "BOTTOM", RESERVE_BOTTOM     }, /* List the bottom matched */
				 { "TOP", RESERVE_TOP	},         /* List the TOP matched */
				 { "MASK", RESERVE_MASK },
                                 { (char *)NULL, (u_short)NULL  }
                               };

void
add_reserve(char *creator, char *resv, char *reason, time_t created)
{
  reserve_t *reserve = leetmalloc(sizeof(reserve_t));
  reserve->next = main_reserve;
  main_reserve = reserve;

  reserve->creator = leetstrdup(creator);
  reserve->resv = leetstrdup(resv);
  reserve->reason = leetstrdup(reason);

    if (created == 0)
    reserve->created = time(NULL);
  else
    reserve->created = created;
  reserve->matches = 0;
}

int
del_reserve(sock_t *sock, char *resv, char *reason)
{
  reserve_t *tmp, **all;
 
  for (all = &main_reserve; *all; all = &((*all)->next))
    if (strcasecmp((*all)->resv, resv) == 0 && strcasecmp((*all)->reason, reason) == 0)
    {
      tell_chans(replies[RPL_REMRESV], sock->name, resv, (*all)->reason);
      leetfree((*all)->creator, strlen((*all)->creator) + 1);
      leetfree((*all)->resv, strlen((*all)->resv) + 1);
      leetfree((*all)->reason, strlen((*all)->reason) + 1);
      tmp = *all;
      *all = (*all)->next;
      leetfree(tmp, sizeof(reserve_t));
      return 1;
    }
  return 0;
}

reserve_t *
find_reserve(char *resv, char *reason)
{
  reserve_t *reserve = main_reserve;
  
  for (; reserve; reserve = reserve->next)
    if (strcasecmp(reserve->resv, resv) == 0 && strcasecmp(reserve->reason, reason) == 0)
      return reserve;
  return NULL;
}

void
del_all_reserves(void) /* make sure we dont save things twice */
{
  reserve_t *tmp, **all = &main_reserve;
            
  while (*all)
  {
    leetfree((*all)->creator, strlen((*all)->creator) + 1);
    leetfree((*all)->resv, strlen((*all)->resv) + 1);
    leetfree((*all)->reason, strlen((*all)->reason) + 1);
    tmp = *all;
    *all = (*all)->next;
    leetfree(tmp, sizeof(reserve_t));
  }
  main_reserve = NULL;
}
 
int
list_reserves(sock_t *sock, char *args, u_short qtype)
{
  reserve_t *reserve = main_reserve;
  char *resv, *reason;
  u_short k = 0;
            
  if (!reserve)
    tosock(sock, replies[ERR_NORESV]);

  resv = strtok(args, " ");
  reason = strtok(NULL, "");
        
  switch (qtype)
  {
    case RESERVE_CNT:
      for (; reserve; reserve = reserve->next)
        k++;
      tosock(sock, replies[RPL_RESVCNT], k);
      break;
    case RESERVE_ALL:
      for (; reserve; reserve = reserve->next)
      {
        tosock(sock, replies[RPL_RESVLISTA], ++k, reserve->resv, reserve->reason,
               reserve->matches);
        tosock(sock, replies[RPL_RESVLISTB], leetctime(reserve->created),
               reserve->creator);
      }
      tosock(sock, replies[RPL_ENDOFRESV]);
      break;
 case RESERVE_TOP:
      if (!args || atoi(args) <= 0)
        return NEED_HELP;
      for (; reserve; reserve = reserve->next) 
        k++;
      if (atoi(args) > k)
        tosock(sock, replies[ERR_TOOMANYRESV], k);
      {
        reserve_t **all = leetcalloc(k + 1, sizeof(reserve_t *));
        short j = k = 0;
        for (reserve = main_reserve; (all[k++] = reserve); reserve = reserve->next)
          ;
        qsort(all, --k, sizeof(reserve_t *), &resvtop);
        for (k = atoi(args); j < k && (reserve = all[j]);)
        {
          tosock(sock, replies[RPL_RESVLISTA], ++j, reserve->resv, reserve->reason,
                 reserve->matches);
          tosock(sock, replies[RPL_RESVLISTB], leetctime(reserve->created),
                 reserve->creator);
        } 
        leetfree(all, (--j * sizeof(reserve_t *)));
      }
      tosock(sock, replies[RPL_ENDOFRESV]);
      break;
 case RESERVE_MASK:
      for (; reserve; reserve = reserve->next)
        if (match(resv, reserve->resv) && match(reason, reserve->reason))
        {
          tosock(sock, replies[RPL_RESVLISTA], ++k, reserve->resv,
                 reserve->reason, reserve->matches);
          tosock(sock, replies[RPL_RESVLISTB], leetctime(reserve->created),
                 reserve->creator);
        }  
      if (k > 0)
        tosock(sock, replies[RPL_ENDOFRESV]);
      else
        tosock(sock, replies[ERR_NOMATCHRESV]);
      break;

 case RESERVE_BOTTOM:
      if (!args || atoi(args) <= 0)
        return NEED_HELP;
      for (; reserve; reserve = reserve->next)
        k++;
      if (atoi(args) > k)
        tosock(sock, replies[ERR_TOOMANYRESV], k);
      {
        reserve_t **all = leetcalloc(k + 1, sizeof(reserve_t *));
        short j = k = 0;
        for (reserve = main_reserve; (all[k++] = reserve); reserve = reserve->next)
          ;
        qsort(all, --k, sizeof(reserve_t *), &resvbottom);
        for (k = atoi(args); j < k && (reserve = all[j]);)
        {
          tosock(sock, replies[RPL_RESVLISTA], ++j, reserve->resv, reserve->reason,
                 reserve->matches);
          tosock(sock, replies[RPL_RESVLISTB], leetctime(reserve->created),
                 reserve->creator);
        }
        leetfree(all, (--j * sizeof(reserve_t *)));
      }
      tosock(sock, replies[RPL_ENDOFRESV]);
      break;
}
  return 1;
}

int   
resvtop(const void *a, const void *b)
{  
  reserve_t *c = (reserve_t *) * (unsigned *)a, *d = (reserve_t *) * (unsigned *)b;
  return d->matches - c->matches;
}  
  
int
resvbottom(const void *a, const void *b)
{         
  reserve_t *c = (reserve_t *) * (unsigned *)a, *d = (reserve_t *) * (unsigned *)b;
  return c->matches - d->matches;
}       
